void
gnome_builder_plugins_init (void)
{
  /* This function is a dummy function that is exported so that
   * we can ensure that --as-needed does not drop linking against
   * this particular shared library.
   */
}
